"""

  Let's first grok our test first:

   >>> grok('infrae.rest.tests.grok.simple')

  Now, if we have a folder we can use our REST component:

    >>> root = getRootFolder()
    >>> _ = root.manage_addProduct['OFS'].manage_addFolder('folder', 'Folder')
    >>> print http('GET /folder/++rest++testsimple HTTP/1.0')
    HTTP/1.0 200 OK
    Content-Length: 7
    Content-Type: plain/text
    <BLANKLINE>
    Nothing

  However we don't have PUT, POST, DELETE, HEAD actions:

    >>> print http('PUT /folder/++rest++testsimple HTTP/1.0')
    Traceback (most recent call last):
       ...
    MethodNotAllowed: PUT

    >>> print http('POST /folder/++rest++testsimple HTTP/1.0')
    Traceback (most recent call last):
       ...
    MethodNotAllowed: POST

    >>> print http('DELETE /folder/++rest++testsimple HTTP/1.0')
    Traceback (most recent call last):
       ...
    MethodNotAllowed: DELETE

    >>> print http('DELETE /folder/++rest++testsimple HTTP/1.0')
    Traceback (most recent call last):
       ...
    MethodNotAllowed: DELETE

    >>> print http('HEAD /folder/++rest++testsimple HTTP/1.0')
    Traceback (most recent call last):
       ...
    MethodNotAllowed: HEAD

  If you test the view on a different object than a folder, it won't
  work as it is registered only for folders:

    >>> _ = root.manage_addProduct['OFS'].manage_addImage('image', 'Image')
    >>> print http('GET /image/++rest++testsimple HTTP/1.0')
    Traceback (most recent call last):
      ...
    NotFound: testsimple

"""

from OFS.Folder import Folder
from five import grok
from infrae.rest import REST


class TestSimple(REST):
    grok.context(Folder)

    def GET(self):
        self.response.setHeader('Content-Type', 'plain/text')
        return "Nothing"



"""

  You can't define a REST view with no methods:

   >>> grok('infrae.rest.tests.grok.norestmethod')
   Traceback (most recent call last):
     ...
   GrokError: REST component testempty doesn't define any REST method

"""

from OFS.Folder import Folder
from five import grok
from infrae.rest import REST


class TestEmpty(REST):
    grok.context(Folder)



# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: components.py 44853 2010-08-30 14:47:20Z sylvain $

from five import grok
from zExceptions import NotFound
from zope import component
from zope.publisher.browser import applySkin
from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.traversing.namespace import view

from infrae.rest.interfaces import MethodNotAllowed, IRESTLayer

import simplejson

ALLOWED_REST_METHODS = ('GET', 'POST', 'HEAD', 'PUT',)


class REST(object):
    """A base REST component
    """
    grok.baseclass()
    grok.implements(IBrowserPublisher)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def response(self):
        return self.request.response

    def browserDefault(self, request):
        """Render the component using a method called the same way
        that the HTTP method name.
        """
        if request.method in ALLOWED_REST_METHODS:
            if hasattr(self, request.method):
                return self, (request.method,)
        raise MethodNotAllowed(request.method)

    def publishTraverse(self, request, name):
        """You can traverse to a method called the same way that the
        HTTP method name.
        """
        if name in ALLOWED_REST_METHODS and name == request.method:
            if hasattr(self, name):
                return getattr(self, name)
        raise NotFound(name)

    def json_response(self, result):
        """Encode a result as a JSON response.
        """
        self.response.setHeader('Content-Type', 'application/json')
        return simplejson.dumps(result)


class MethodNotAllowedView(grok.View):
    grok.context(MethodNotAllowed)
    grok.name('error.html')

    def update(self):
        self.response.setStatus(405)

    def render(self):
        return u"Method not allowed"


class RESTNamespace(view):
    """Implement a namespace ++rest++.
    """

    def traverse(self, name, ignored):
        self.request.shiftNameToApplication()
        applySkin(self.request, IRESTLayer)
        if name:
            view = component.queryMultiAdapter(
                (self.context, self.request), name=name)
            if view is None:
                raise NotFound(name)
            # Set view parent/name for security
            view.__name__ = name
            view.__parent__ = self.context
            return view
        return self.context



"""

  Let's first grok our test first:

   >>> grok('infrae.rest.tests.grok.require')

  Now, if we have a folder we can use our REST component:

    >>> root = getRootFolder()
    >>> root.manage_addProduct['OFS'].manage_addFolder('folder', 'Folder')

  And try to access it like this:

    >>> print http('GET /folder/++rest++secret HTTP/1.0')
    HTTP/1.0 401 Unauthorized
    Content-Length: 0
    Www-Authenticate: basic realm="Zope"
    <BLANKLINE>

  Ok if we authenticate ourselves it will work:

    >>> print http('GET /folder/++rest++secret HTTP/1.0\\r\\n'
    ...      'Authorization: Basic manager:manager')
    HTTP/1.0 200 OK
    Content-Length: 13
    Content-Type: text/html;charset=utf-8
    <BLANKLINE>
    <p>Secret</p>

"""

from OFS.Folder import Folder
from five import grok
from infrae.rest import REST


class TestSecret(REST):
    grok.context(Folder)
    grok.name('secret')
    grok.require('zope2.ViewManagementScreens')

    def GET(self):
        return "<p>Secret</p>"



# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: interfaces.py 42329 2010-05-25 08:28:32Z sylvain $

from zope.publisher.interfaces.http import IHTTPRequest


class IRESTLayer(IHTTPRequest):
    """To this layer are registered the REST components.
    """


class MethodNotAllowed(Exception):
    """This method is not allowed.
    """




"""

  Let's first grok our test first:

   >>> grok('infrae.rest.tests.grok.json')

  Now, if we have a folder we can use our REST component:

    >>> root = getRootFolder()
    >>> root.manage_addProduct['OFS'].manage_addFolder('folder', 'Folder')

    >>> print http('GET /folder/++rest++testjson HTTP/1.0')
    HTTP/1.0 200 OK
    Content-Length: 32
    Content-Type: application/json
    <BLANKLINE>
    {"integer": 42, "boolean": true}

  You can optionaly provide values:

    >>> print http('PUT /folder/++rest++testjson HTTP/1.0')
    HTTP/1.0 200 OK
    Content-Length: 20
    Content-Type: application/json
    <BLANKLINE>
    {"identifier": null}

    >>> print http('PUT /folder/++rest++testjson HTTP/1.0\\r\\n'
    ...     'Content-Length: 13\\r\\n'
    ...     'Content-Type: application/x-www-form-urlencoded\\r\\n\\r\\n'
    ...     'identifier=42')
    HTTP/1.0 200 OK
    Content-Length: 20
    Content-Type: application/json
    <BLANKLINE>
    {"identifier": "42"}

    >>> print http('HEAD /folder/++rest++testjson HTTP/1.0')
    HTTP/1.0 204 No Content
    Content-Length: 0
    <BLANKLINE>
    <BLANKLINE>

"""

from OFS.Folder import Folder
from five import grok
from infrae.rest import REST


class TestJSon(REST):
    grok.context(Folder)

    def GET(self):
        return self.json_response({'boolean': True, 'integer': 42})

    def PUT(self, identifier=None):
        return self.json_response({'identifier': identifier})

    def HEAD(self):
        return None




# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: setup.py 45745 2010-10-07 13:20:32Z sylvain $

from setuptools import setup, find_packages
import os

version = '1.0.1'

tests_require = [
    'infrae.wsgi [test]',
    ]


setup(name='infrae.rest',
      version=version,
      description="Define a REST API to access and manage Silva content",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Framework :: Zope2",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='silva cms rest api zope',
      author='Infrae',
      author_email='info@infrae.com',
      url='http://infrae.com/products/silva',
      license='BSD',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['infrae',],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'five.grok',
        'martian',
        'setuptools',
        'simplejson',
        'zope.component',
        'zope.interface',
        'zope.publisher',
        'zope.traversing',
        ],
      tests_require = tests_require,
      extras_require = {'test': tests_require},
      )



# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: meta.py 42404 2010-05-26 17:13:48Z sylvain $

from App.class_init import InitializeClass as initializeClass
from Products.Five.security import protectName

from five import grok
from martian.error import GrokError
from zope import interface, component
import martian

from infrae.rest.components import REST, ALLOWED_REST_METHODS
from infrae.rest.interfaces import IRESTLayer


def default_view_name(factory, module=None, **data):
    return factory.__name__.lower()


class RESTGrokker(martian.ClassGrokker):
    """Grok REST views.
    """
    martian.component(REST)
    martian.directive(grok.context)
    martian.directive(grok.name, get_default=default_view_name)
    martian.directive(grok.require, name='permission')

    def execute(self, factory, context, name, permission, config, **kw):
        """Register the REST component as a view on the IREST layer.
        """
        if permission is None:
            permission = 'zope.Public'
        method_defined = False
        for method in ALLOWED_REST_METHODS:
            if hasattr(factory, method):
                method_defined = True
        if not method_defined:
            raise GrokError(
                "REST component %s doesn't define any REST method" % (name,),
                factory)

        adapts = (context, IRESTLayer)
        config.action(
            discriminator=('adapter', adapts, interface.Interface, name),
            callable=component.provideAdapter,
            args=(factory, adapts, interface.Interface, name))

        for method in ALLOWED_REST_METHODS:
            if hasattr(factory, method):
                config.action(
                    discriminator = ('five:protectName', factory, method),
                    callable = protectName,
                    args = (factory, method, permission))

        config.action(
            discriminator = ('five:initialize:class', factory),
            callable = initializeClass,
            args = (factory,))
        return True



# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: test_grok.py 42404 2010-05-26 17:13:48Z sylvain $

import unittest
import doctest

import infrae.rest
from infrae.wsgi.testing import BrowserLayer, suite_from_package, http


class RestLayer(BrowserLayer):
    default_users = {
        'manager': ['Manager'],
        }


layer = RestLayer(infrae.rest)
globs = {'http': http,
         'getRootFolder': layer.get_application,
         'grok': layer.grok,}


def create_test(build_test_suite, name):
    test = build_test_suite(
        name,
        globs=globs,
        optionflags=doctest.ELLIPSIS + doctest.NORMALIZE_WHITESPACE)
    test.layer = layer
    return test


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(suite_from_package('infrae.rest.tests.grok', create_test))
    return suite
